
/**
  *
  *  Computer Networks Course
  *  PA0 Included Content
  *
  *  Logger Class (thread-safe)
  *
  *  Author: Kamyar Allahverdi
  *
  */

#ifndef __LOGGER_H__
#define __LOGGER_H__


#include <iostream>
#include <fstream>
#include <pthread.h>

//coloring is just supported for console output
#define RED "31"
#define GREEN "32"
#define YELLOW "33"
#define BLUE "34"
#define WHITE "37"

//undef to output to console
//#define FILE_LOGGING

//universal lock for all objects, just log one item at the same time
pthread_mutex_t log_mutex = PTHREAD_MUTEX_INITIALIZER;

class Logger
{
public:
    enum Level { Debug, Error, Info };

    std::ostream *os;
    bool bold;
    const char *colorcode;

    Logger(Level l,std::ostream &oss=std::cout){
    	setLevel(l);
    	os=&oss;
    }

    void setLevel(Level l){
    	switch(l){
    		case Debug:
    			bold=true;
    			colorcode = YELLOW;
    			break;

    		case Error:
    			bold=true;
    			colorcode = RED;
    			break;

    		case Info:
    			bold=false;
    			colorcode = WHITE;
    			break;
    	}
    }

    Logger& operator<<(std::string& text){
      pthread_mutex_lock( &log_mutex );
    	if ( os == &std::cout )
  			(*os) << "\033[" << (bold ? "1" : "0") << ";" << colorcode << "m" << text.c_str()
  				<< "\033[0m";
  		else
  			(*os) << text.c_str();
      os->flush();
      pthread_mutex_unlock( &log_mutex );
      return *this;
    }

    Logger& operator<<(const char *text){
      pthread_mutex_lock( &log_mutex );
    	if ( os == &std::cout )
  			(*os) << "\033[" << (bold ? "1" : "0") << ";" << colorcode << "m" << text
  				<< "\033[0m";
  		else
  			(*os) << text;
  		os->flush();
      pthread_mutex_unlock( &log_mutex );
    	return *this;
    }
    
    Logger& operator<<(const int num){
      pthread_mutex_lock( &log_mutex );
    	if ( os == &std::cout )
  			(*os) << "\033[" << (bold ? "1" : "0") << ";" << colorcode << "m" << num
  				<< "\033[0m";
  		else
  			(*os) << num;
  		os->flush();
      pthread_mutex_unlock( &log_mutex );
      return *this;
    }

    // to support std::endl
	typedef std::ostream& (*manip)( std::ostream& );
    Logger& operator<<(manip fp){
      pthread_mutex_lock( &log_mutex ); 
      (*os) << fp;
      pthread_mutex_unlock( &log_mutex );
      return *this;
    }
};

#ifdef FILE_LOGGING
  //file logging, random names, to avoid overwriting
  //when running to version from the same binary
  char *gen_random(char s[], const int from, const int len) {
      srand(time(0));
      static const char alphanum[] =
          "0123456789"
          "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
          "abcdefghijklmnopqrstuvwxyz";

      for (int i = from; i < from+len; ++i) {
          s[i] = alphanum[rand() % (sizeof(alphanum) - 1)];
      }
      //for full verison, not part modifier
      //s[len] = 0;
      return s;
  }
  char logname[] = "log_random.txt";
  std::ofstream logfile(gen_random(logname,4,6));
  Logger LOG(Logger::Info,logfile);
  Logger DEBUG(Logger::Debug,logfile);
  Logger ERR(Logger::Error,logfile);
#else
  //console debugging
  Logger LOG(Logger::Info);
  Logger DEBUG(Logger::Debug);
  Logger ERR(Logger::Error);
#endif

#endif