
/**
  *
  *  Computer Networks Course
  *  PA0 Included Content
  *
  *  Author: Kamyar Allahverdi
  *  
  *  Loading TGA code borrowed from Jeff Molofee (NeHe)
  *
  */

#ifndef __PA0_MAP__
#define __PA0_MAP__

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#ifdef __APPLE__
#include <GLUT/glut.h>
#else
#include <GL/glut.h>
#endif

#define SCREEN_WIDTH 640
#define SCREEN_HEIGHT 640

#define SIDEBAR_WIDTH 200

typedef void (*draw_func)();
typedef void (*mouse_func)(int button, int state, int x, int y);
typedef void (*keyboard_func)(unsigned char key, int x, int y);

typedef struct
{
	GLubyte *imageData;
	GLuint bpp;
	GLuint width;
	GLuint height;
	GLuint texID;
} TextureImage;

void draw_cell(int width)
{
	glBegin(GL_POLYGON);
	 	glTexCoord2f(0.0f, 1.0f); glVertex2f(0, 0);
	 	glTexCoord2f(0.0f, 0.0f); glVertex2f(0, width);
	 	glTexCoord2f(1.0f, 0.0f); glVertex2f(width, width);
	 	glTexCoord2f(1.0f, 1.0f); glVertex2f(width, 0);
	glEnd();
}

void glprint(GLfloat x, GLfloat y, const char* text)
{
    glPushMatrix();
    glTranslatef(x, y, 0);
    glScalef(0.15, -0.15, 0.15);
    for( const char* p = text; *p; p++)
    {
        glutStrokeCharacter(GLUT_STROKE_ROMAN, *p);
    }
    glPopMatrix();
}


// Loads A TGA File Into Memory
bool LoadTGA(TextureImage *texture, const char *filename)
{    
	GLubyte		TGAheader[12]={0,0,2,0,0,0,0,0,0,0,0,0};
	GLubyte		TGAcompare[12];
	GLubyte		header[6];
	GLuint		bytesPerPixel;
	GLuint		imageSize;
	GLuint		temp;
	GLuint		type=GL_RGBA;
	FILE *file = fopen(filename, "rb");
	
	if(	file==NULL ||
	   fread(TGAcompare,1,sizeof(TGAcompare),file)!=sizeof(TGAcompare) ||
	   memcmp(TGAheader,TGAcompare,sizeof(TGAheader))!=0		||
	   fread(header,1,sizeof(header),file)!=sizeof(header))
	{
		if (file == NULL)
			return false;
		else
		{
			fclose(file);
			return false;
		}
	}
	texture->width  = header[1] * 256 + header[0];
	texture->height = header[3] * 256 + header[2];
    
 	if(	texture->width	<=0	||
	   texture->height	<=0	||
	   (header[4]!=24 && header[4]!=32))
	{
		fclose(file);
		return false;
	}
	texture->bpp	= header[4];
	bytesPerPixel	= texture->bpp/8;
	imageSize	= texture->width*texture->height*bytesPerPixel;
	texture->imageData=(GLubyte *)malloc(imageSize);
	
	if(	texture->imageData==NULL ||
	   fread(texture->imageData, 1, imageSize, file)!=imageSize)
	{
		if(texture->imageData!=NULL)
			free(texture->imageData);
		
		fclose(file);
		return false;
	} 

	// Swap The 1st And 3rd Bytes ('R'ed and 'B'lue)
	for(GLuint i=0; i<int(imageSize); i+=bytesPerPixel)
	{								
		temp=texture->imageData[i];
		texture->imageData[i] = texture->imageData[i + 2];
		texture->imageData[i + 2] = temp;
	}
	
	fclose (file);
	
	// Build A Texture From The Data
	glGenTextures(1, &texture[0].texID);
	
	glBindTexture(GL_TEXTURE_2D, texture[0].texID);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	if (texture[0].bpp==24)
	{
		type=GL_RGB;
	}
	glTexImage2D(GL_TEXTURE_2D, 0, type, texture[0].width, texture[0].height, 0, type, GL_UNSIGNED_BYTE, texture[0].imageData);
	
	return true;
}

#endif
